spool 911_CreacionTablas.log
 
 
  CREATE TABLE PAISES 
   (NOMBRE_PAIS VARCHAR2(28 BYTE) NOT NULL ENABLE, 
	  CONSTRAINT PAISES_PK PRIMARY KEY (NOMBRE_PAIS)
   );
 

  CREATE TABLE REGIONES 
   (NOMBRE_REGION VARCHAR2(28 BYTE) NOT NULL ENABLE, 
	  NOMBRE_PAIS VARCHAR2(28 BYTE) NOT NULL ENABLE, 
	  CONSTRAINT REGIONES_PK PRIMARY KEY (NOMBRE_REGION), 
	  CONSTRAINT REGIONES_FK1 FOREIGN KEY (NOMBRE_PAIS)
    REFERENCES PAISES (NOMBRE_PAIS) ENABLE
   );

 
 CREATE TABLE CIUDADES 
   (NOMBRE_CIUDAD VARCHAR2(28 BYTE) NOT NULL ENABLE, 
	  NOMBRE_REGION VARCHAR2(28 BYTE) NOT NULL ENABLE, 
	  CONSTRAINT CIUDADES_PK PRIMARY KEY (NOMBRE_CIUDAD, NOMBRE_REGION),
    CONSTRAINT CIUDADES_FK1 FOREIGN KEY (NOMBRE_REGION)
	  REFERENCES REGIONES (NOMBRE_REGION) ENABLE
   );
   

  CREATE TABLE TIENDAS 
   (TIENDA_ID NUMBER(*,0) NOT NULL ENABLE, 
	  NOMBRE_CIUDAD VARCHAR2(28 BYTE) NOT NULL ENABLE, 
	  NOMBRE_REGION VARCHAR2(28 BYTE) NOT NULL ENABLE, 
	  CORREO_GERENTE VARCHAR2(28 BYTE), 
	  FRANQUICIA NUMBER(1,0) DEFAULT 0 NOT NULL ENABLE, --1 Franquicia, 0 propia 
	  VIRTUAL NUMBER(1,0) DEFAULT 0 NOT NULL ENABLE,  -- 1 Virtual, 0 fsica
	  DIRECCION_POSTAL VARCHAR2(28 BYTE), 
	  CONSTRAINT TIENDAS_PK PRIMARY KEY (TIENDA_ID),
    CONSTRAINT TIENDAS_CHK1 CHECK (FRANQUICIA in (0,1)) ENABLE, 
	  CONSTRAINT TIENDAS_CHK2 CHECK (VIRTUAL in (0,1)) ENABLE, 
	  CONSTRAINT TIENDAS_FK1 FOREIGN KEY (NOMBRE_CIUDAD, NOMBRE_REGION)
    REFERENCES CIUDADES (NOMBRE_CIUDAD, NOMBRE_REGION) ENABLE
   ); 
      

  CREATE TABLE FECHAS 
   (FECHA DATE NOT NULL ENABLE, 
	  HORA NUMBER(2,0) NOT NULL ENABLE, 
	  CONSTRAINT FECHAS_PK PRIMARY KEY (FECHA, HORA),
    CONSTRAINT "FECHAS_CHK1" CHECK (HORA BETWEEN 0 AND 23) ENABLE
   );
   
  
  CREATE TABLE GENEROS 
   (GENERO VARCHAR2(15 BYTE) NOT NULL ENABLE, 
	  CONSTRAINT GENEROS_PK PRIMARY KEY (GENERO),
    CONSTRAINT GENEROS_CHK1 CHECK (GENERO in ('Hombre', 'Mujer', 'Unisex', 'Nio', 'Nia')) ENABLE
   );
   
   
  CREATE TABLE TIPOS 
   (TIPO VARCHAR2(15 BYTE) NOT NULL ENABLE, 
	  CONSTRAINT TIPOS_PK PRIMARY KEY (TIPO)
   );
   
   
  CREATE TABLE PRODUCTOS 
   (COD_EAN NUMBER(13,0) NOT NULL ENABLE, 
	  DESCRIPCION VARCHAR2(50 BYTE), 
	  FECHA_INCORPORACION DATE NOT NULL ENABLE, 
	  TIPO VARCHAR2(15 BYTE) NOT NULL ENABLE, 
	  GENERO VARCHAR2(15 BYTE) NOT NULL ENABLE, 
	  CONSTRAINT PRODUCTOS_PK PRIMARY KEY (COD_EAN),
    CONSTRAINT PRODUCTOS_FK1 FOREIGN KEY (TIPO)
	  REFERENCES TIPOS (TIPO) ENABLE, 
	  CONSTRAINT PRODUCTOS_FK2 FOREIGN KEY (GENERO)
	  REFERENCES GENEROS (GENERO) ENABLE
   );
   

  CREATE TABLE VENTAS 
   (VENTA_ID NUMBER NOT NULL ENABLE, 
	  TIENDA_ID NUMBER(*,0) NOT NULL ENABLE, 
	  COD_EAN NUMBER(*,0) NOT NULL ENABLE, 
	  FECHA DATE NOT NULL ENABLE, 
	  HORA NUMBER(2,0) NOT NULL ENABLE, 
	  UD_VENDIDAS NUMBER(*,0) NOT NULL ENABLE, 
	  CTD_TOTAL NUMBER(*,0) NOT NULL ENABLE, -- Cantidad total de producto vendido ese da y hora
	  PB_TOTAL NUMBER(10,2) NOT NULL ENABLE, -- Precio bruto total de los productos (con el mismo EAN) vendidos en ese da y hora
	  BN_TOTAL NUMBER(10,2) NOT NULL ENABLE, -- Beneficio neto total de los productos (con el mismo EAN) vendidos en ese da y hora
	  PB_UNITARIO NUMBER(10,2) NOT NULL ENABLE, -- Precio bruto unitario de la venta
	  BN_UNITARIO NUMBER(10,2), -- Beneficio neto unitario de la venta
	  CONSTRAINT VENTAS_PK PRIMARY KEY (VENTA_ID, TIENDA_ID, COD_EAN, FECHA, HORA),
    CONSTRAINT VENTAS_FK3 FOREIGN KEY (FECHA, HORA)
	  REFERENCES FECHAS (FECHA, HORA) ENABLE, 
	  CONSTRAINT VENTAS_FK1 FOREIGN KEY (TIENDA_ID)
	  REFERENCES TIENDAS (TIENDA_ID) ON DELETE CASCADE ENABLE, -- Se permite la eliminacin de una tienda con ventas asociadas pues no interesa
    --mantener registros si se cierra una tienda
	  CONSTRAINT VENTAS_FK2 FOREIGN KEY (COD_EAN)
	  REFERENCES PRODUCTOS (COD_EAN) ON DELETE CASCADE ENABLE  -- Se permite la eliminacin de un producto con ventas asociadas pues no interesa
    --mantener registros si se da de baja el producto
   );   


  CREATE TABLE TIEMPOS 
   (ANO NUMBER(4,0) DEFAULT 0 NOT NULL ENABLE, 
	  MES NUMBER(2,0) DEFAULT 0 NOT NULL ENABLE, 
	  CONSTRAINT TIEMPOS_PK PRIMARY KEY (ANO, MES),
    CONSTRAINT TIEMPOS_CHK1 CHECK (MES BETWEEN 1 AND 12) ENABLE
   );
   
   
  CREATE TABLE N_EMPLEADOS 
   (TIENDA_ID NUMBER(*,0) NOT NULL ENABLE, 
	  ANO NUMBER(4,0) DEFAULT 0 NOT NULL ENABLE, 
	  MES NUMBER(2,0) DEFAULT 0 NOT NULL ENABLE, 
	  NUMERO_EMPLEADOS NUMBER(6,0) NOT NULL ENABLE, 
	  CONSTRAINT N_EMPLEADOS_CHK1 CHECK (MES BETWEEN 1 AND 12) ENABLE, 
	  CONSTRAINT N_EMPLEADOS_CHK2 CHECK (NUMERO_EMPLEADOS >0) ENABLE, 
	  CONSTRAINT N_EMPLEADOS_PK PRIMARY KEY (TIENDA_ID, ANO, MES),
    CONSTRAINT N_EMPLEADOS_FK1 FOREIGN KEY (ANO, MES)
	  REFERENCES TIEMPOS (ANO, MES) ENABLE
   ); 


 CREATE TABLE ESTADISTICAS_DIRECCION 
   (EST_ID NUMBER NOT NULL ENABLE, 
	  BN_AGREGADO NUMBER(10,2), -- Beneficio neto de toda la cadena 
	  ID_TIENDA_MAX NUMBER(*,0), -- Tienda con el mximo beneficio neto 
	  BN_TIENDA_MAX NUMBER NOT NULL ENABLE, --Beneficio de la tienda con mximo beneficio neto 
    COD_EAN_MAX NUMBER(13,0), --Cdigo EAN13 del producto ms vendido
	  CTD_UNIDADES_EAN_MAX NUMBER, -- Unidades del producto ms vendido 
	  H_MAX NUMBER, --Hora del da en la que se han producido ms ventas 
	  CTD_UNIDADES_H_MAX NUMBER, --Cantidad de unidades vendidas en la hora en que se han producido ms ventas 
	  H_MIN NUMBER, --Hora del da en la que se han producido menos ventas 
	  CTD_UNIDADES_H_MIN NUMBER, --Cantidad de unidades vendidas en la hora en que se han producido menos ventas 
	  D_MAX NUMBER, --Da del mes en la que se han producido ms ventas 
	  CTD_UNIDADES_D_MAX NUMBER, --Cantidad de unidades vendidas en el da del mes en que se han producido ms ventas 
	  D_MIN NUMBER, --Da del mes en la que se han producido menos ventas 
	  CTD_UNIDADES_D_MIN NUMBER, --Cantidad de unidades vendidas en el da del mes en que se han producido menos ventas 
	  CIUDAD_MAX VARCHAR2(20 BYTE), --Ciudad donde ms beneficios se han obtenido 
	  BN_CIUDAD_MAX NUMBER(10,2),  --Beneficio neto de la ciudad donde ms beneficios se han obtenido
	  PORCNTJ_BENFCS_VIRTLS NUMBER, --Porcentaje de beneficios de las tiendas virutales sobre el total de beneficios de la cadena
    CONSTRAINT TABLE1_PK PRIMARY KEY (EST_ID)
   );
   
   
  CREATE TABLE LOGS 
   (LOG_ID NUMBER(10,0) NOT NULL ENABLE, 
	  NOMBRE_PROCEDIMIENTO VARCHAR2(30 BYTE), 
	  PARAMETROS_ENTRADA VARCHAR2(200 BYTE), 
	  PARAMETROS_SALIDA VARCHAR2(200 BYTE), 
	  FECHA DATE, 
	  CONSTRAINT LOG_PK PRIMARY KEY (LOG_ID)
   );
   
   
spool off;