/*
	NOMBRE DEL PAQUETE: PKG_EST_DIR
	
  FUNCIONALIDAD: Proporciona las funciones necesarias para el mantenimiento de
  las estadsticas para direccin.
*/

spool 915_Implementacion_PKG_EST_DIR.log;

------------------------------------------------
------------------- CABECERA -------------------
------------------------------------------------

CREATE OR REPLACE PACKAGE PKG_EST_DIR IS
	NOMPAQUETE CHAR(20) := 'PKG_EST_DIR';
  

/*
Busca el nombre de la ciudad en la que est ubicada una tienda.
El procedimiento queda registrado en la tabla de LOGS.
@param e_tienda_id: El identificador de la tienda de la que se quiere saber su ciudad.
@return el nombre de la ciudad donde se ubica la tienda. Si no existe valor devuelve una cadena vaca.
*/
FUNCTION CIUDAD_DE_TIENDA(
  e_tienda_id IN NUMBER
)
RETURN VARCHAR2;  


/*
Calcula el ratio de beneficio de las tiendas fsicas sobre el total de beneficios de la cadena.
para un periodo determinado.
El procedimiento queda registrado en la tabla de LOGS.
@param e_periodo: El periodo de clculo expresado en YYYY. Si es para todos los aos indicar 0.
@return el ratio de beneficio de las tiendas fsicas sobre el total de beneficios de la cadena expresado en "tanto por uno".
*/
FUNCTION CAL_PORC_VIRT (
    e_periodo IN NUMBER
)
RETURN NUMBER;


/*
Busca la ciudad con mximo beneficios para un periodo determinado que recibe como parmetro. Si hay dos ciudades con el mismo beneficio
slo devuelve los datos de una de ellas.
El procedimiento queda registrado en la tabla de LOGS.
@param e_periodo: El periodo de clculo expresado en YYYY. Si es para todos los aos indicar 0.
@return ciudad: La ciudad con el mximo beneficio para ese periodo. NULL en caso de que no haya ventas.
@return bn_ciudad: El beneficio neto correspondiente a la ciudad. 0 en caso de que no haya ventas.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE BUSCAR_CIUDAD_BN_MAX(
    e_periodo IN NUMBER
,   ciudad OUT VARCHAR2
,   bn_ciudad OUT NUMBER
,   rsp OUT STRING
);


/*
Busca el da con menos ventas para un periodo determinado y devuelve el nmero de da y las unidades vendidas para ese da.
El procedimiento queda registrado en la tabla de LOGS.
@param e_periodo: El periodo de clculo expresado en YYYY. Si es para todos los aos indicar 0.
@return dia: El da con menos ventas para ese periodo. 0 en caso de que no haya ventas.
@return unidades: Unidades vendidas correspondientes a ese da. 0 en caso de que no haya ventas.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE BUSCAR_D_UD_MIN(
   e_periodo IN NUMBER
,  dia OUT NUMBER
,  unidades OUT NUMBER
,  rsp OUT STRING
);


/*
Busca el da con ms ventas para un periodo determinado y devuelve el nmero de da y las unidades vendidas para ese da.
El procedimiento queda registrado en la tabla de LOGS.
@param e_periodo: El periodo de clculo expresado en YYYY. Si es para todos los aos indicar 0.
@return dia: El da con ms ventas para ese periodo. 0 en caso de que no haya ventas.
@return unidades: Unidades vendidas correspondientes a ese da. 0 en caso de que no haya ventas.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE BUSCAR_D_UD_MAX (
   e_periodo IN NUMBER
,  dia OUT NUMBER
,  unidades OUT NUMBER
,  rsp OUT STRING
);


/*
Busca la hora con menos ventas para un periodo determinado y devuelve la hora y las unidades vendidas para esa hora.
El procedimiento queda registrado en la tabla de LOGS.
@param e_periodo: El periodo de clculo expresado en YYYY. Si es para todos los aos indicar 0.
@return hora: La hora con menos ventas para ese periodo. 0 en caso de que no haya ventas.
@return unidades: Unidades vendidas correspondientes a esa hora. 0 en caso de que no haya ventas.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE BUSCAR_H_UD_MIN (
   e_periodo IN NUMBER
,  hora OUT NUMBER
,  unidades OUT NUMBER
,  rsp OUT STRING
);


/*
Busca la hora con ms ventas para un periodo determinado y devuelve la hora y las unidades vendidas para esa hora.
El procedimiento queda registrado en la tabla de LOGS.
@param e_periodo: El periodo de clculo expresado en YYYY. Si es para todos los aos indicar 0.
@return hora: La hora con ms ventas para ese periodo. 0 en caso de que no haya ventas.
@return unidades: Unidades vendidas correspondientes a esa hora. 0 en caso de que no haya ventas.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE BUSCAR_H_UD_MAX (
   e_periodo IN NUMBER
,  hora OUT NUMBER
,  unidades OUT NUMBER
,  rsp OUT STRING
);


/*
Busca el producto con ms unidades vendidas para un periodo determinado y devuelve el cdigo del producto y las unidades vendidas para ese producto.
El procedimiento queda registrado en la tabla de LOGS.
@param e_periodo: El periodo de clculo expresado en YYYY. Si es para todos los aos indicar 0.
@return cod_ean: El cdigo del producto con ms ventas para ese periodo. Cadena vaca en caso de que no haya ventas.
@return unidades: Unidades vendidas correspondientes a ese producto. 0 en caso de que no haya ventas.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE BUSCAR_PDT_UD_MAX (
   e_periodo IN NUMBER
,  cod_ean OUT NUMBER
,  unidades OUT NUMBER
,  rsp OUT STRING
);


/*
Busca la tienda con mayor beneficio para un periodo determinado y devuelve el cdigo de la tienda y el beneficio generado por esa tienda.
El procedimiento queda registrado en la tabla de LOGS.
@param e_periodo: El periodo de clculo expresado en YYYY. Si es para todos los aos indicar 0.
@return tienda_id: El cdigo de la tienda con ms beneficio para ese periodo. 0 en caso de que no haya ventas.
@return bn_tienda: Beneficio correspondientes a esa tienda. 0 en caso de que no haya ventas.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE BUSCAR_TDA_BN_MAX (
   e_periodo IN NUMBER
,  tienda_id OUT NUMBER
,  bn_tienda OUT NUMBER
,  rsp OUT STRING
);


/*
Indica mediante un cdigo si una tienda es virtual o no. 
1 Es virtual.
0 No es virtual.
2 Si la tienda no existe.
El procedimiento queda registrado en la tabla de LOGS.
@param e_tienda_id: El cdigo de la tienda del que se quiere conocer si es virtual o no.
@return NUMBER indicando si la tienda es virtual o fsica o si la tienda no existe.
*/
FUNCTION ES_VIRTUAL(
    e_tienda_id IN NUMBER
)
RETURN NUMBER;

/*
Inserta una fila en la tabla ESTADISTICAS_DIRECCION con los valores estadsticos necesarios. Esta tabla tiene una fila para cada periodo anual identificada por el ao con el formato YYYY. 
Adems tiene una lnea identificada con 0 para almacenar los valores estadsticos globales. Por tanto, al insertar una lnea es condicin que no existan ventas y este procedimiento tomar los valores
que recibe como parmetro como la nica venta hasta el momento para dicho periodo.
El procedimiento queda registrado en la tabla de LOGS.
No confirma las modificaciones en la Base de Datos, NO COMMIT INSIDE.
@param   e_periodo: El periodo para el que se quiere insertar una lnea.
@param   e_tienda_id: La tienda que hace la venta.
@param   e_cod_ean: el producto vendido.
@param   e_dia: El da en que se produce la venta.
@param   e_hora: La hora en que se produce la venta.
@param   e_ud_vendidas:  Las unidades vendidas.
@param   e_precio_bruto: El precio bruto unitario del producto vendido.
@param   e_beneficio_neto: El beneficio neto unitario del producto vendido.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR. No se ha podido insertar nueva lnea de estadstica.' Si no se puede insertar la lnea.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE INSRTR_LIN_EST_D  (
    e_periodo IN NUMBER
,   e_tienda_id IN NUMBER
,   e_cod_ean IN NUMBER
,   e_dia IN NUMBER
,   e_hora IN NUMBER
,   e_ud_vendidas IN NUMBER
,   e_precio_bruto IN NUMBER
,   e_beneficio_neto IN NUMBER
,   rsp OUT STRING
);


/*
Actualiza la lnea de la tabla ESTADISTICAS_DIRECCION con los valores que recibe como parmetro de entrada.
El procedimiento queda registrado en la tabla de LOGS.
No confirma las modificaciones en la Base de Datos, NO COMMIT INSIDE.
@param e_periodo: El periodo cuya lnea se quiere modificar.
@param valores: Dato con los valores actualizados de la lnea a modificar.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR. No se ha podido actualizar la estadstica.' Si no se ha podido actualizar la tabla.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE ACTLZR_LIN_EST_D  (
   e_periodo IN NUMBER
,  valores IN ESTADISTICAS_DIRECCION%ROWTYPE
,  rsp OUT STRING
);


/*
Proporciona un cursor SYS_REFCURSOR sobre la tabla ESTADISTICAS_DIRECCION considerando el tipo de datos que se requieren.
Si se requieren todas las filas de la tabla hay que indicar 'GLOBAL' como primer parmetro de entrada. Si se requiere un ao concreto
hay que indicar 'INDIVIDUAL' como primer parmetro de entrada.
El procedimiento queda registrado en la tabla de LOGS.
@param e_tipo: El tipo de datos que se requiere. 'INDIVIDUAL' o 'GLOBAL'.
@param e_ao: El ao para el que se quieren los datos. Si se ha indicado 'INDIVIDUAL' no es relevante y no se tiene en cuenta.
@return cursor SYS_REFCURSOR sobre la tabla ESTADISTICAS_DIRECCION.
*/
FUNCTION OBTENER_CURSOR(
   e_tipo IN VARCHAR2
,  e_ao IN NUMBER
)
RETURN SYS_REFCURSOR ;


/*
Actualiza la lnea de la tabla ESTADISTICAS_DIRECCION con los valores que recibe como parmetro de entrada. Tiene dos posibilidades, actualizar la lnea
correspondiente a los datos globales (identificada en la tabla con el id 0) o la lnea correspondiente a un ao en concreto (identificada en la tabla con el id YYYY).
Para la actualizacin de los datos globales hay que indicar 'TOTAL' como parmetro de entrada en e_tipo. 
Para la actualizacin de los datos de un ao hay que indicar 'ANUAL' como parmetro de entrada en e_tipo.
El procedimiento queda registrado en la tabla de LOGS.
No confirma las modificaciones en la Base de Datos, NO COMMIT INSIDE.
@param   e_tipo: 'TOTAL'  para actualizar la estadstica global, 'ANUAL' para actualizar una estadstica en concreto.
@param   e_tienda_id: La tienda que genera la actualizacin a travs de una venta.
@param   e_cod_ean: El producto que genera la actualizacin a travs de una venta.
@param   e_fecha: La fecha de la venta que motiva la actualizacin.
@param   e_hora: La hora de la venta que motiva la actualizacin.
@param   e_ud_vendidas: Las unidades de la venta que motiva la actualizacin.
@param   e_precio_bruto: El precio bruto unitario de la venta que motiva la actualizacin.
@param   e_beneficio_neto: El beneficio neto unitario de la venta que motiva la actualizacin.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR.'ERROR. No se ha podido insertar nueva lnea de estadstica.' Si no se ha podido actualizar la tabla.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE ACTLZR_EST (
   e_tipo VARCHAR2
,  e_tienda_id IN NUMBER
,  e_cod_ean IN NUMBER
,  e_fecha IN DATE
,  e_hora IN NUMBER
,  e_ud_vendidas IN NUMBER  
,  e_precio_bruto IN NUMBER 
,  e_beneficio_neto IN NUMBER
,  rsp OUT STRING);


/*
Actualiza la lnea de la tabla ESTADISTICAS_DIRECCION correspondiente a los datos globales con los valores que recibe como parmetro de entrada. 
Para ello utiliza el procedimiento ACTLRZR_EST indicando como tipo de actualizacin 'GLOBAL'.
El procedimiento queda registrado en la tabla de LOGS.
No confirma las modificaciones en la Base de Datos, NO COMMIT INSIDE.
@param   e_tienda_id: La tienda que genera la actualizacin a travs de una venta.
@param   e_cod_ean: El producto que genera la actualizacin a travs de una venta.
@param   e_fecha: La fecha de la venta que motiva la actualizacin.
@param   e_hora: La hora de la venta que motiva la actualizacin.
@param   e_ud_vendidas: Las unidades de la venta que motiva la actualizacin.
@param   e_precio_bruto: El precio bruto unitario de la venta que motiva la actualizacin.
@param   e_beneficio_neto: El beneficio neto unitario de la venta que motiva la actualizacin.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE ACTLZR_EST_TOTAL (
   e_tienda_id IN NUMBER
,  e_cod_ean IN NUMBER
,  e_fecha IN DATE
,  e_hora IN NUMBER
,  e_ud_vendidas IN NUMBER  
,  e_precio_bruto IN NUMBER 
,  e_beneficio_neto IN NUMBER
,  rsp OUT STRING
)
;


/*
Actualiza la lnea de la tabla ESTADISTICAS_DIRECCION correspondiente a un ao determinado con los valores que recibe como parmetro de entrada. 
Para ello utiliza el procedimiento ACTLRZR_EST indicando como tipo de actualizacin 'ANUAL'.
El procedimiento queda registrado en la tabla de LOGS.
No confirma las modificaciones en la Base de Datos, NO COMMIT INSIDE.
@param   e_tienda_id: La tienda que genera la actualizacin a travs de una venta.
@param   e_cod_ean: El producto que genera la actualizacin a travs de una venta.
@param   e_fecha: La fecha de la venta que motiva la actualizacin.
@param   e_hora: La hora de la venta que motiva la actualizacin.
@param   e_ud_vendidas: Las unidades de la venta que motiva la actualizacin.
@param   e_precio_bruto: El precio bruto unitario de la venta que motiva la actualizacin.
@param   e_beneficio_neto: El beneficio neto unitario de la venta que motiva la actualizacin.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE ACTLZR_EST_ANUAL (
   e_tienda_id IN NUMBER
,  e_cod_ean IN NUMBER
,  e_fecha IN DATE
,  e_hora IN NUMBER
,  e_ud_vendidas IN NUMBER  
,  e_precio_bruto IN NUMBER 
,  e_beneficio_neto IN NUMBER
,  rsp OUT STRING
);


/*
Actualiza todas las filas de la tabla ESTADISTICAS_DIRECCION.
No confirma las modificaciones en la Base de Datos, NO COMMIT INSIDE.
El procedimiento queda registrado en la tabla de LOGS.
No tiene parmetros de entrada.
@return rsp: 'Ok' si la ejecucin es correcta.
             'ERROR. No se puede guardar el log de la transaccin' si no se puede guardar el registro de log.
             'ERROR + Tipo de error' en otro caso.
*/
PROCEDURE ACTLZR_EST_GLOBAL (
   rsp OUT STRING
);


/*
Calcula en beneficio neto agregado de la cadena para un periodo determinado.
El procedimiento queda registrado en la tabla de LOGS.
@param e_periodo: El periodo para el que se quiere tener el dato.
@return el beneficio neto agregado de la cadena para el periodo.
*/
FUNCTION CAL_BN_AGREGADO (
    e_periodo IN NUMBER
) RETURN NUMBER;

END PKG_EST_DIR;
/

----------------------------------------------
------------------- CUERPO -------------------
----------------------------------------------
CREATE OR REPLACE PACKAGE BODY PKG_EST_DIR IS   
  
FUNCTION CIUDAD_DE_TIENDA(
  e_tienda_id IN NUMBER
)

RETURN VARCHAR2 

AS

   ciudad VARCHAR2 (28);

BEGIN

   SELECT NOMBRE_CIUDAD INTO ciudad FROM TIENDAS
   WHERE TIENDA_ID = e_tienda_id;

   RETURN ciudad;
  
   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       ciudad := '';
     WHEN OTHERS THEN
       RAISE;
       
END CIUDAD_DE_TIENDA;


FUNCTION CAL_PORC_VIRT (
    e_periodo IN NUMBER
) RETURN NUMBER

AS 
    tipo_tienda1 NUMBER;
    tipo_tienda2 NUMBER;
    bn_t1 NUMBER;
    bn_t2 NUMBER;
    porcentaje NUMBER;
    crs SYS_REFCURSOR;

BEGIN
 
      -- Si la consulta se hace para todos los periodos se toma cursor sobre todas las ventas de cada tipo de tienda. Si se concreta algn ao
      -- se toma cursor sobre las ventas de cada tipo de tienda del ao. Se toma tambin el valor del indicador de tienda virtual para cada venta.
      IF e_periodo = 0 THEN
          OPEN crs FOR SELECT * FROM (SELECT t.VIRTUAL, SUM (v.UD_VENDIDAS * v.BN_UNITARIO) AS BENEFICIO
                                      FROM VENTAS v
                                      JOIN
                                      TIENDAS t
                                      ON v.TIENDA_ID = t.TIENDA_ID
                                      GROUP BY t.VIRTUAL);
      ELSE
          OPEN crs FOR SELECT * FROM (SELECT t.VIRTUAL, SUM (v.UD_VENDIDAS * v.BN_UNITARIO) AS BENEFICIO
                                      FROM VENTAS v
                                      JOIN
                                      TIENDAS t
                                      ON v.TIENDA_ID = t.TIENDA_ID
                                      WHERE TO_CHAR (FECHA, 'YYYY') = e_periodo
                                      GROUP BY t.VIRTUAL);
      END IF;
   
   FETCH crs INTO tipo_tienda1, bn_t1;
   IF crs%NOTFOUND THEN
      porcentaje :=0; -- si no hay ventas para el periodo, el % es 0.
   ELSE
      FETCH crs INTO tipo_tienda2, bn_t2;
      IF crs%NOTFOUND THEN
        -- el 0 es tienda fsica, el 1 es tienda virtual.     
        IF tipo_tienda1 = 0 THEN porcentaje :=0; -- si no hay ms valore y ha tomado valor para 0, son tiendas fsicas y por tanto % es 0.
        ELSE porcentaje := 1; -- si slo hay beneficio de tiendas fsicas entonces el porcentaje es un 100%.
        END IF;
      ELSE -- si ha encontrado valor en los dos hay que calcular el beneficio.
        IF tipo_tienda1 = 0 THEN porcentaje := bn_t2/(bn_t2+bn_t1); -- si el primero son las fsicas.
        ELSE porcentaje := bn_t1/(bn_t2+bn_t1); -- si no son las fsicas, es que son las virtuales.
        END IF;
      END IF;
   END IF;
    
   CLOSE crs;
   
   RETURN porcentaje;

END CAL_PORC_VIRT ;


PROCEDURE BUSCAR_CIUDAD_BN_MAX(
    e_periodo IN NUMBER
,   ciudad OUT VARCHAR2
,   bn_ciudad OUT NUMBER
,   rsp OUT STRING
)

AS 

   crs SYS_REFCURSOR;
  
   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.  
   id_log := PKG_CONTROL.INSERTA_LOG('BUSCAR_CIUDAD_BN_MAX', 'Periodo: '||e_periodo); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con las operaciones.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      IF e_periodo = 0 THEN
          OPEN crs FOR SELECT * FROM (SELECT t.NOMBRE_CIUDAD AS CIUDAD,  SUM(v.BN_UNITARIO * v.UD_VENDIDAS) AS TOTAL_BENEFICIO
                                      FROM VENTAS v
                                      JOIN 
                                      TIENDAS T
                                      ON v.TIENDA_ID = T.TIENDA_ID
                                      GROUP BY t.NOMBRE_CIUDAD
                                      ORDER BY TOTAL_BENEFICIO DESC)
                                      WHERE ROWNUM = 1;

      ELSE 

          OPEN crs FOR SELECT * FROM (SELECT t.NOMBRE_CIUDAD AS CIUDAD,  SUM(v.BN_UNITARIO * v.UD_VENDIDAS) AS TOTAL_BENEFICIO
                                      FROM VENTAS v
                                      JOIN
                                      TIENDAS T
                                      ON v.TIENDA_ID = T.TIENDA_ID
                                      WHERE TO_CHAR(FECHA, 'YYYY') = e_periodo
                                      GROUP BY t.NOMBRE_CIUDAD
                                      ORDER BY TOTAL_BENEFICIO DESC)
                                      WHERE ROWNUM = 1;

      END IF;
   END IF;
 
 
      FETCH crs INTO ciudad, bn_ciudad;
      -- puede ser que no haya ventas y por tanto no se devuelvan valores.
      IF crs%NOTFOUND THEN
          ciudad :='';
          bn_ciudad :=0;
      END IF;
 
      CLOSE crs;
      
      -- Registra el final de la ejecucin en la tabla de LOGS.      
      rsp := ('OK');
      PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);  
      
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario.
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin';
       -- Para cualquier otro error informa en la tabla de LOG del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END BUSCAR_CIUDAD_BN_MAX;


PROCEDURE BUSCAR_D_UD_MIN(
   e_periodo IN NUMBER
,  dia OUT NUMBER
,  unidades OUT NUMBER
,  rsp OUT STRING
)

AS 

   crs SYS_REFCURSOR;
   
   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('BUSCAR_D_UD_MIN', 'Periodo: '||e_periodo); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE    
      IF e_periodo = 0 THEN 
          OPEN crs FOR SELECT * FROM (SELECT TO_CHAR(FECHA, 'DD') AS DIA, SUM (UD_VENDIDAS) AS TOTAL_UNIDADES FROM ventas
                                      GROUP BY TO_CHAR(FECHA, 'DD')
                                      ORDER BY TOTAL_UNIDADES)
                       WHERE ROWNUM = 1;
      ELSE
          OPEN crs FOR SELECT * FROM (SELECT TO_CHAR(FECHA, 'DD') AS DIA, SUM (UD_VENDIDAS) AS TOTAL_UNIDADES FROM ventas
                                      WHERE TO_CHAR (FECHA, 'YYYY') = e_periodo
                                      GROUP BY TO_CHAR(FECHA, 'DD')
                                      ORDER BY TOTAL_UNIDADES)
                        WHERE ROWNUM = 1;
      END IF;
   END IF;
 
   FETCH crs INTO dia, unidades;
   
   -- Si no encuentra valores es que no hay ventas para es periodo y da 0 y 0.
   IF crs%NOTFOUND THEN
      dia := 0;
      unidades :=0;
   END IF;
   
   CLOSE crs;
   
   -- Registra el final de la ejecucin en la tabla de LOGS.      
   rsp := 'OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);  
      
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin';
       --Para cualquier otro error informa en la tabla de LOG del error
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
         
END BUSCAR_D_UD_MIN;


PROCEDURE BUSCAR_D_UD_MAX (
   e_periodo IN NUMBER
,  dia OUT NUMBER
,  unidades OUT NUMBER
,  rsp OUT STRING
)

AS 
  
   crs SYS_REFCURSOR;
  
   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('BUSCAR_D_UD_MAX', 'Periodo: '||e_periodo); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
     IF e_periodo = 0 THEN
          OPEN crs FOR SELECT * FROM (SELECT TO_CHAR(FECHA, 'DD') AS DIA, SUM (UD_VENDIDAS) AS TOTAL_UNIDADES FROM ventas
                                      GROUP BY TO_CHAR(FECHA, 'DD')
                                      ORDER BY TOTAL_UNIDADES DESC)
                       WHERE ROWNUM = 1;
     ELSE
          OPEN crs FOR SELECT * FROM (SELECT TO_CHAR(FECHA, 'DD') AS DIA, SUM (UD_VENDIDAS) AS TOTAL_UNIDADES FROM ventas
                                      WHERE TO_CHAR (FECHA, 'YYYY') = e_periodo   
                                      GROUP BY TO_CHAR(FECHA, 'DD')
                                      ORDER BY TOTAL_UNIDADES DESC)
                       WHERE ROWNUM = 1;
     END IF;
   END IF;
 
   FETCH crs INTO dia, unidades;
   -- Si no encuentra valores es que no hay ventas para es periodo y da 0 y 0.   
   IF crs%NOTFOUND THEN
      dia := 0;
      unidades := 0;
   END IF;
 
   CLOSE crs;
   
   -- Registra el final de la ejecucin en la tabla de LOGS.      
   rsp := 'OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);  
   
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin';
       --Para cualquier otro error informa en la tabla de LOG del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);   

END BUSCAR_D_UD_MAX;


PROCEDURE BUSCAR_H_UD_MIN (
   e_periodo IN NUMBER
,  hora OUT NUMBER
,  unidades OUT NUMBER
,  rsp OUT STRING
)

AS 

   crs SYS_REFCURSOR;

   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('BUSCAR_H_UD_MIN', 'Periodo: '||e_periodo); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      IF e_periodo = 0 THEN 
          OPEN crs FOR SELECT * FROM (SELECT HORA, SUM(UD_VENDIDAS) AS TOTAL_UNIDADES FROM ventas
                                      GROUP BY HORA
                                      ORDER BY TOTAL_UNIDADES)
                       WHERE ROWNUM = 1;
      ELSE
          OPEN crs FOR SELECT * FROM (SELECT HORA, SUM(UD_VENDIDAS) AS TOTAL_UNIDADES FROM ventas
                                      WHERE TO_CHAR (FECHA, 'YYYY') = e_periodo
                                      GROUP BY HORA
                                      ORDER BY TOTAL_UNIDADES)
                       WHERE ROWNUM = 1;
      END IF;
   END IF;
 
   FETCH crs INTO hora, unidades;
   -- Si no encuentra valores es que no hay ventas para es periodo y da 0 y 0. 
   IF crs%NOTFOUND THEN
      hora := 0;
      unidades := 0;
   END IF;
 
   CLOSE crs;
   
   -- Registra el final de la ejecucin en la tabla de LOGS.      
   rsp := ('OK');
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);  

   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       --Para cualquier otro error informa en la tabla de LOG del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);   
   

END BUSCAR_H_UD_MIN;


PROCEDURE BUSCAR_H_UD_MAX (
   e_periodo IN NUMBER
,  hora OUT NUMBER
,  unidades OUT NUMBER
,  rsp OUT STRING
)

AS 

   crs SYS_REFCURSOR;
   
   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('BUSCAR_H_UD_MAX', 'Periodo: '||e_periodo); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE 
      IF e_periodo = 0 THEN
          OPEN crs FOR SELECT * FROM (SELECT HORA, SUM(UD_VENDIDAS) AS TOTAL_UNIDADES FROM ventas
                                      GROUP BY HORA
                                      ORDER BY TOTAL_UNIDADES DESC)
                       WHERE ROWNUM = 1; 
      ELSE
          OPEN crs FOR SELECT * FROM (SELECT HORA, SUM(UD_VENDIDAS) AS TOTAL_UNIDADES FROM ventas
                                      WHERE TO_CHAR (FECHA, 'YYYY') = e_periodo
                                      GROUP BY HORA
                                      ORDER BY TOTAL_UNIDADES DESC);
      END IF;
   END IF;   
 
   FETCH crs INTO hora, unidades;
   -- Si no encuentra valores es que no hay ventas para es periodo y da 0 y 0. 
   IF crs%NOTFOUND THEN
      hora := 0;
      unidades := 0;
   END IF;
   CLOSE crs;
   
   -- Registra el final de la ejecucin en la tabla de LOGS.      
   rsp := ('OK');
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);  

   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin';
       --Para cualquier otro error informa en la tabla de LOG del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
         
END BUSCAR_H_UD_MAX;


PROCEDURE BUSCAR_PDT_UD_MAX (
   e_periodo IN NUMBER
,  cod_ean OUT NUMBER
,  unidades OUT NUMBER
,  rsp OUT STRING
)

AS 

   crs SYS_REFCURSOR;

   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('BUSCAR_PDT_UD_MAX', 'Periodo: '||e_periodo); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      IF e_periodo = 0 THEN
          OPEN crs FOR SELECT * FROM (SELECT COD_EAN, SUM (UD_VENDIDAS) AS TOTAL_UNIDADES FROM ventas
                                      GROUP BY COD_EAN
                                      ORDER BY TOTAL_UNIDADES DESC)
                                      WHERE ROWNUM = 1;
      ELSE
          OPEN crs FOR SELECT * FROM (SELECT COD_EAN, SUM (UD_VENDIDAS) AS TOTAL_UNIDADES FROM ventas
                                      WHERE TO_CHAR (FECHA, 'YYYY') = e_periodo
                                      GROUP BY COD_EAN
                                      ORDER BY TOTAL_UNIDADES DESC)
                       WHERE ROWNUM = 1;
      END IF;
   END IF;
  
   FETCH crs INTO cod_ean, unidades;
   -- Si no encuentra valores es que no hay ventas para es periodo y da cadena vaca y 0. 
   IF crs%NOTFOUND THEN
      cod_ean := '';
      unidades := 0;
   END IF;
   
   CLOSE crs;
   
   -- Registra el final de la ejecucin en la tabla de LOGS.      
   rsp := ('OK');
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);  
   
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin';
       --Para cualquier otro error informa en la tabla de LOG del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);   
   
END BUSCAR_PDT_UD_MAX;


PROCEDURE BUSCAR_TDA_BN_MAX (
   e_periodo IN NUMBER
,  tienda_id OUT NUMBER
,  bn_tienda OUT NUMBER
,  rsp OUT STRING
)

AS 
 
   crs SYS_REFCURSOR;
    
   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('BUSCAR_TDA_BN_MAX', 'Periodo: '||e_periodo); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE    
      IF e_periodo = 0 THEN
           OPEN crs FOR SELECT * FROM (SELECT TIENDA_ID, SUM (UD_VENDIDAS * BN_UNITARIO) AS BENEFICIO 
                                        FROM VENTAS
                                        GROUP BY TIENDA_ID
                                        ORDER BY BENEFICIO DESC)
                         WHERE ROWNUM = 1;
               
      ELSE
           OPEN crs FOR SELECT * FROM (select TIENDA_ID, SUM (UD_VENDIDAS * BN_UNITARIO) AS BENEFICIO FROM ventas
                                       WHERE TO_CHAR (FECHA, 'YYYY') = e_periodo
                                       GROUP BY TIENDA_ID
                                       ORDER BY BENEFICIO DESC)
                                       WHERE ROWNUM = 1;
      END IF;
   END IF;
 
   FETCH crs INTO tienda_id, bn_tienda;
   -- Si no encuentra valores es que no hay ventas para es periodo y da 0 y 0. 
   IF crs%NOTFOUND THEN
      tienda_id := 0;
      bn_tienda := 0;
   END IF;
   
   CLOSE crs;
   
   -- Registra el final de la ejecucin en la tabla de LOGS.      
   rsp := 'OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);  
   
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin';
       --Para cualquier otro error informa en la tabla de LOG del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);   

END BUSCAR_TDA_BN_MAX;


FUNCTION ES_VIRTUAL(
    e_tienda_id IN NUMBER
)
RETURN NUMBER
AS

   rdo NUMBER;

BEGIN

   SELECT VIRTUAL INTO rdo FROM TIENDAS
   WHERE TIENDA_ID = e_tienda_id;

   RETURN rdo;

   EXCEPTION
     WHEN NO_DATA_FOUND THEN
       rdo:=2;
     WHEN OTHERS THEN
       RAISE;
      
END ES_VIRTUAL;


PROCEDURE INSRTR_LIN_EST_D  (
    e_periodo IN NUMBER
,   e_tienda_id IN NUMBER
,   e_cod_ean IN NUMBER
,   e_dia IN NUMBER
,   e_hora IN NUMBER
,   e_ud_vendidas IN NUMBER
,   e_precio_bruto IN NUMBER
,   e_beneficio_neto IN NUMBER
,   rsp OUT STRING
)

AS 

   bn NUMBER; -- Registro del beneficio neto.
   ciudad VARCHAR2(28); -- Registro de ciudad con ms ventas.
   porcentaje NUMBER; -- Registro de ratio beneficio tiendas virtuales sobre total.
   virtual NUMBER; -- Registro para saber si la tienda es virtual o fsica.

   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;
   -- Excepcin en el caso de que falle la insercin.
   problema_insercion EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('INSRTR_LIN_EST_D', 'Periodo: '||e_periodo||'Tienda: '||e_tienda_id||', Producto: '
   ||e_cod_ean||', Dia: '||e_dia||', Hora: '||e_hora||', Unidades: '||e_ud_vendidas||', Precio bruto: '||e_precio_bruto
   ||', Beneficio neto: '||e_beneficio_neto); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE 
      -- Calcula el beneficio neto de la venta
      bn := e_ud_vendidas * e_beneficio_neto;

      -- Los datos vienen de una venta por lo que si la tienda no existe ya ha lanzado error. Por tanto la tienda existe y
      -- dado que ciudad es obligatorio no puede haber una tienda sin ciudad y la SELECT siempre devuelve un valor.
      SELECT NOMBRE_CIUDAD INTO ciudad FROM TIENDAS
      WHERE TIENDA_ID = e_tienda_id;

      -- Almacena el valor sobre si la tienda es virtual o no. 
      virtual := ES_VIRTUAL (e_tienda_id);
      
      IF virtual = 0 THEN -- es tienda fsica.
          porcentaje := 0;
      ELSIF virtual = 1 THEN -- es tienda virtual.
          porcentaje := 1;
      END IF;
 
      -- Inserta la lnea con los valores calculados.
      INSERT INTO ESTADISTICAS_DIRECCION(EST_ID, BN_AGREGADO, ID_TIENDA_MAX, BN_TIENDA_MAX, COD_EAN_MAX,
      CTD_UNIDADES_EAN_MAX, H_MAX, CTD_UNIDADES_H_MAX, H_MIN, CTD_UNIDADES_H_MIN, D_MAX, CTD_UNIDADES_D_MAX,
      D_MIN, CTD_UNIDADES_D_MIN, CIUDAD_MAX, BN_CIUDAD_MAX, PORCNTJ_BENFCS_VIRTLS)
      VALUES (e_periodo, bn, e_tienda_id, bn, e_cod_ean, e_ud_vendidas, e_hora, e_ud_vendidas, e_hora, e_ud_vendidas,
      e_dia, e_ud_vendidas, e_dia, e_ud_vendidas, ciudad, bn, porcentaje);
      
      IF SQL%ROWCOUNT = 0 then
        RAISE problema_insercion;
      end if;
      
      -- Registra el final de la ejecucin en la tabla de LOGS.            
      rsp:='OK';
      PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
      
   END IF;

   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin';
       -- Si no puede insertar la fila.
       WHEN problema_insercion THEN
         rsp:='ERROR. No se ha podido insertar nueva lnea de estadstica.';
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
       --Para cualquier otro error informa en la tabla de LOG del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
         
END INSRTR_LIN_EST_D;


PROCEDURE ACTLZR_LIN_EST_D  (
   e_periodo IN NUMBER
,  valores IN ESTADISTICAS_DIRECCION%ROWTYPE
,  rsp OUT STRING
)

AS 

   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;
   -- Excepcin en el caso de que falle la insercin de la lnea.
   problema_actualizacion EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('ACTLZR_LIN_EST_D', 'Periodo: '||e_periodo); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
     UPDATE ESTADISTICAS_DIRECCION SET
            BN_AGREGADO = valores.BN_AGREGADO,
            ID_TIENDA_MAX = valores.ID_TIENDA_MAX,
            BN_TIENDA_MAX = valores.BN_TIENDA_MAX,
            COD_EAN_MAX = valores.COD_EAN_MAX,
            CTD_UNIDADES_EAN_MAX = valores.CTD_UNIDADES_EAN_MAX,
            H_MAX = valores.H_MAX,
            CTD_UNIDADES_H_MAX = valores.CTD_UNIDADES_H_MAX,
            H_MIN = valores.H_MIN,
            CTD_UNIDADES_H_MIN = valores.CTD_UNIDADES_H_MIN,
            D_MAX = valores.D_MAX,
            CTD_UNIDADES_D_MAX = valores.CTD_UNIDADES_D_MAX,
            D_MIN = valores.D_MIN,
            CTD_UNIDADES_D_MIN = valores.CTD_UNIDADES_D_MIN,
            CIUDAD_MAX = valores.CIUDAD_MAX,
            BN_CIUDAD_MAX = valores.BN_CIUDAD_MAX,
            PORCNTJ_BENFCS_VIRTLS = valores.PORCNTJ_BENFCS_VIRTLS
     WHERE EST_ID = e_periodo;
    
     IF SQL%ROWCOUNT = 0 then
        RAISE problema_actualizacion;
     END IF;

      -- Registra el final de la ejecucin en la tabla de LOGS.            
      rsp:='OK';
      PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
      
   END IF;

   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       WHEN problema_actualizacion THEN
         rsp:='ERROR. No se ha podido actualizar la estadstica.';
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
       --Para cualquier otro error informa en la tabla de LOG del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
         
END ACTLZR_LIN_EST_D;


FUNCTION OBTENER_CURSOR(
   e_tipo IN VARCHAR2
,  e_ao IN NUMBER
)
RETURN SYS_REFCURSOR 

AS 

   crs sys_refcursor;
   problema_cursor EXCEPTION;

BEGIN
  
   IF e_tipo = 'GLOBAL' THEN
     OPEN crs for SELECT * FROM ESTADISTICAS_DIRECCION;
   ELSIF e_tipo = 'INDIVIDUAL' THEN
     OPEN crs FOR SELECT * FROM ESTADISTICAS_DIRECCION
                  WHERE EST_ID = e_ao;
   END IF;
  
  return crs;
  
  EXCEPTION
      WHEN OTHERS THEN
        RAISE problema_cursor;
    
END OBTENER_CURSOR;


PROCEDURE ACTLZR_EST (
   e_tipo VARCHAR2
,  e_tienda_id IN NUMBER
,  e_cod_ean IN NUMBER
,  e_fecha IN DATE
,  e_hora IN NUMBER
,  e_ud_vendidas IN NUMBER  
,  e_precio_bruto IN NUMBER 
,  e_beneficio_neto IN NUMBER
,  rsp OUT STRING)

AS

   -- Varialbe para almacenar el periodo del que se quieren actualizar los datos.
   periodo NUMBER;

   -- Cursor para leer los datos del periodo consultado.
   crs SYS_REFCURSOR;
           
   -- Variables para reoger los datos del cursor.
   datos ESTADISTICAS_DIRECCION%ROWTYPE;
   resultado_venta NUMBER;
   bn_t_fisicas NUMBER;
   bn_t_virtuales NUMBER;
   -- Variable para poder llamar a los procedimientos internos.
   rsp_temp STRING (200);

   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepciones.
   problema_log EXCEPTION;
   problema_insercion EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('ACTLZR_EST', 'Tipo: '||e_tipo||', Tienda: '||e_tienda_id||', Producto: '
   ||e_cod_ean||', Dia: '||e_fecha||', Hora: '||e_hora||', Unidades: '||e_ud_vendidas||', Precio bruto: '||e_precio_bruto
   ||', Beneficio neto: '||e_beneficio_neto); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  

      -- Segn sea el tipo recibido tomar la lnea con el valor 0 (GLOBAL) o la del ao correspondiente (ANUAL)
      IF e_tipo = 'ANUAL' THEN
          periodo := TO_CHAR (e_fecha, 'YYYY');
      ELSIF e_tipo = 'TOTAL' THEN
          periodo := 0;
      END IF;
    
      -- Obtener cursor.
      crs := OBTENER_CURSOR('INDIVIDUAL',periodo);
      
      FETCH crs INTO datos;
    
      -- Si no existe la fila para el ao, la crea con los datos calculados conforme a la venta que ha llamado el procedimiento.
      IF crs%NOTFOUND THEN
    
          
          INSRTR_LIN_EST_D(periodo, e_tienda_id, e_cod_ean, TO_NUMBER (TO_CHAR(e_fecha,'DD')), e_hora, e_ud_vendidas, e_precio_bruto, e_beneficio_neto, rsp_temp);
          IF sql%ROWCOUNT = 0 THEN 
            RAISE problema_insercion;
          END IF;
       
      -- Si la lnea existe debe actualizarla.
      ELSE
    
          --Calcula el resultado de la venta actual.
          resultado_venta := e_ud_vendidas * e_beneficio_neto;    
        
          --beneficio neto de la cadena es el beneficio que tena registrado ms el de la nueva venta.
          datos.BN_AGREGADO := datos.BN_AGREGADO + resultado_venta;
          
          -- Calcula la tienda que ms beneficios ha conseguido y su beneficio. 
          -- Si la tienda que hace la venta es la misma que ya estaba mantiene el id y cambia el importe de beneficio aadiendo al anterior el de la nueva venta.
          -- Si es otra tienda realiza la bsqueda en la tabla de ventas y actualiza ambos valores.
          IF e_tienda_id = datos.ID_TIENDA_MAX THEN 
              datos.BN_TIENDA_MAX := datos.BN_TIENDA_MAX + resultado_venta;
          ELSE
              BUSCAR_TDA_BN_MAX(periodo, datos.ID_TIENDA_MAX, datos.BN_TIENDA_MAX, rsp_temp);
          END IF;
        
          -- Calcula el producto ms vendido y la cantidad de unidades vendidas
          -- Si el producto es el mismo que ya estaba mantiene el cdigo EAN13 y cambia el importe de las unidades vendidas aadiendo al anterior las unidades de la nueva venta.
          -- Si es otro producto realiza la bsqueda en la tabla de ventas y actualiza ambos valores.
          IF e_cod_ean = datos.COD_EAN_MAX THEN
              datos.CTD_UNIDADES_EAN_MAX := datos.CTD_UNIDADES_EAN_MAX + e_ud_vendidas;
          ELSE
              BUSCAR_PDT_UD_MAX(periodo,datos.COD_EAN_MAX, datos.CTD_UNIDADES_EAN_MAX, rsp_temp); 
          END IF;
    
          -- Calcula la hora del da con ms ventas y la cantidad de unidades vendidas.
          -- Si la hora del da es la misma que ya estaba la mantiene y cambia el importe de las unidades vendidas aadiendo al anterior las unidades de la nueva venta.
          -- So es otra hora, realiza la bsqueda en la tabla de ventas y actualiza ambos valores.
          IF e_hora = datos.H_MAX THEN
              datos.CTD_UNIDADES_H_MAX := datos.CTD_UNIDADES_H_MAX + e_ud_vendidas;
          ELSE
              BUSCAR_H_UD_MAX(periodo, datos.H_MAX, datos.CTD_UNIDADES_H_MAX, rsp_temp);
          END IF;
    
         -- Calcula la hora del da con menos ventas y la cantidad de unidades vendidas
          BUSCAR_H_UD_MIN (periodo, datos.H_MIN, datos.CTD_UNIDADES_H_MIN, rsp_temp);
    
          -- Calcula el da con ms ventas y la cantidad de unidades vendidas.
          -- Si el da es el mismo que ya estaba lo mantiene y cambia el importe de las unidades vendidas aadiendo al anterior las unidades de la nueva venta.
          -- So es otro dia, realiza la bsqueda en la tabla de ventas y actualiza ambos valores.
          IF TO_CHAR(e_fecha, 'DD') = datos.D_MAX THEN
              datos.CTD_UNIDADES_D_MAX := datos.CTD_UNIDADES_D_MAX + e_ud_vendidas;
          ELSE
             BUSCAR_D_UD_MAX (periodo, datos.D_MAX, datos.CTD_UNIDADES_D_MAX, rsp_temp);
          END IF;
        
         -- Calcula la hora del da con menos ventas y la cantidad de unidades vendidas
          BUSCAR_D_UD_MIN (periodo, datos.D_MIN, datos.CTD_UNIDADES_D_MIN, rsp_temp);
              
          -- Calcula la ciudad con ms ventas y su beneficio.
          -- Si la ciudad que hace la venta es la misma que ya estaba la mantiene y cambia el importe de beneficio aadiendo al anterior el de la nueva venta.
          -- Si es otra ciudad, realiza la bsqueda en la tabla de ventas y actualiza ambos valores.
          IF CIUDAD_DE_TIENDA(e_tienda_id) = datos.CIUDAD_MAX THEN
            datos.BN_CIUDAD_MAX := datos.BN_CIUDAD_MAX + resultado_venta;
          ELSE
            BUSCAR_CIUDAD_BN_MAX (periodo, datos.CIUDAD_MAX, datos.BN_CIUDAD_MAX, rsp_temp);
          END IF;

         -- Calcula el porcentaje de beneficios de las virtuales respecto del resto
          datos.PORCNTJ_BENFCS_VIRTLS := CAL_PORC_VIRT(periodo);
    
          -- Actualiza la lnea correspondiente con los valores calculados.
          ACTLZR_LIN_EST_D (periodo, datos, rsp_temp);
    
        END IF;
               
      -- Cierra el cursor
      CLOSE crs;
    
   END IF;
   
   -- Registra el final de la ejecucin en la tabla de LOGS.
   rsp:='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       -- Si no se ha podido insertar la fila.
       WHEN problema_insercion THEN
         rsp:='ERROR. No se ha podido insertar nueva lnea de estadstica.';
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
       --Para cualquier otro error informa en la tabla de LOG del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END ACTLZR_EST;


PROCEDURE ACTLZR_EST_TOTAL (
   e_tienda_id IN NUMBER
,  e_cod_ean IN NUMBER
,  e_fecha IN DATE
,  e_hora IN NUMBER
,  e_ud_vendidas IN NUMBER  
,  e_precio_bruto IN NUMBER 
,  e_beneficio_neto IN NUMBER
,  rsp OUT STRING
)

AS

   rsp_temp STRING (200);
   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('ACTLZR_EST_TOTAL','Tienda: '||e_tienda_id||', Producto: '
   ||e_cod_ean||', Fecha: '||e_fecha||', Hora: '||e_hora||', Unidades: '||e_ud_vendidas||', Precio bruto: '||e_precio_bruto
   ||', Beneficio neto: '||e_beneficio_neto); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      ACTLZR_EST ('TOTAL', e_tienda_id ,e_cod_ean, e_fecha, e_hora, e_ud_vendidas, e_precio_bruto, e_beneficio_neto, rsp_temp);
   END IF;

   -- Registra el final de la ejecucin en la tabla de LOGS.   
   rsp:='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
   
   
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       --Para cualquier otro error informa en la tabla de LOG del error
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);   
  
END ACTLZR_EST_TOTAL;


PROCEDURE ACTLZR_EST_ANUAL (
   e_tienda_id IN NUMBER
,  e_cod_ean IN NUMBER
,  e_fecha IN DATE
,  e_hora IN NUMBER
,  e_ud_vendidas IN NUMBER  
,  e_precio_bruto IN NUMBER 
,  e_beneficio_neto IN NUMBER
,  rsp OUT STRING
)

AS

   rsp_temp STRING (200);
   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Excepcin en el caso de que falle el registro de LOG.
   problema_log EXCEPTION;
   
BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento.   
   id_log := PKG_CONTROL.INSERTA_LOG('ACTLZR_EST_ANUAL','Tienda: '||e_tienda_id||', Producto: '
   ||e_cod_ean||', Fecha: '||e_fecha||', Hora: '||e_hora||', Unidades: '||e_ud_vendidas||', Precio bruto: '||e_precio_bruto
   ||', Beneficio neto: '||e_beneficio_neto); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      ACTLZR_EST ('ANUAL', e_tienda_id ,e_cod_ean, e_fecha, e_hora, e_ud_vendidas, e_precio_bruto, e_beneficio_neto, rsp_temp);
   END IF;
   
   -- Registra el final de la ejecucin en la tabla de LOGS.   
   rsp:='OK';
   PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
   
   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       --Para cualquier otro error informa en la tabla de LOG del error
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);   
  
END ACTLZR_EST_ANUAL;


PROCEDURE ACTLZR_EST_GLOBAL (
   rsp OUT STRING
)
AS 

   crs SYS_REFCURSOR;

   datos ESTADISTICAS_DIRECCION%ROWTYPE;

   -- Variable para almacenar el valor del LOG_ID que le corresponda a la llamada al procedimiento.
   id_log INTEGER;
   -- Variable para poder llamar a los procedimientos internos.
   rsp_temp STRING (200);
   -- Excepcin en el caso de que falle el registro de LOG
   problema_log EXCEPTION;

BEGIN

   -- Registra en la tabla de LOG la llamada al procedimiento   
   id_log := PKG_CONTROL.INSERTA_LOG('ACTLZR_EST_GLOBAL',''); 

   -- Si hay algn problema al insertar el LOG lanza una excepcin definida por el usuario.
   -- Si no hay problema al insertar el LOG prosigue con la insercin.
   IF id_log = 0 THEN
      RAISE problema_log;
   
   ELSE  
      
      -- Obtiene el cursor con todas las filas de la tabla ESTADISTICAS_DIRECCION.
      crs := OBTENER_CURSOR('GLOBAL',0); --da igual que le ponga 0 o cualquier valor pues no afecta.

      LOOP
        FETCH crs INTO datos;
        EXIT WHEN crs%NOTFOUND;
        -- Calcula el Beneficio neto agregado
        datos.BN_AGREGADO := CAL_BN_AGREGADO(datos.EST_ID);
        -- Calcula la tienda que ms beneficios a conseguido y su beneficio.         
        BUSCAR_TDA_BN_MAX (datos.EST_ID, datos.ID_TIENDA_MAX, datos.BN_TIENDA_MAX,rsp_temp);
        -- Calcula el producto ms vendido y la cantidad de unidades vendidas        
        BUSCAR_PDT_UD_MAX (datos.EST_ID, datos.COD_EAN_MAX, datos.CTD_UNIDADES_EAN_MAX,rsp_temp);
        -- Calcula la hora del da con mas ventas y la cantidad de unidades vendidas        
        BUSCAR_H_UD_MAX (datos.EST_ID, datos.H_MAX, datos.CTD_UNIDADES_H_MAX,rsp_temp);
        -- Calcula la hora del da con menos ventas y la cantidad de unidades vendidas        
        BUSCAR_H_UD_MIN (datos.EST_ID, datos.H_MIN, datos.CTD_UNIDADES_H_MIN,rsp_temp);        
        -- Calcula el da con ms ventas y la cantidad de unidades vendidas.        
        BUSCAR_D_UD_MAX (datos.EST_ID, datos.D_MAX, datos.CTD_UNIDADES_D_MAX,rsp_temp);        
        -- Calcula el da con menos ventas y la cantidad de unidades vendidas.        
        BUSCAR_D_UD_MIN (datos.EST_ID, datos.D_MIN, datos.CTD_UNIDADES_D_MIN,rsp_temp);
        -- Calcula la ciudad con ms ventas y su beneficio.        
        BUSCAR_CIUDAD_BN_MAX (datos.EST_ID, datos.CIUDAD_MAX, datos.BN_CIUDAD_MAX,rsp_temp);
        -- Calcula el porcentaje de beneficios de las virtuales respecto del resto        
        datos.PORCNTJ_BENFCS_VIRTLS := CAL_PORC_VIRT (datos.EST_ID);  
        -- Actualiza la lnea correspondiente con los valores calculados.       
        ACTLZR_LIN_EST_D (datos.EST_ID, datos, rsp_temp);
               
      END LOOP;
      
      -- Cierra el cursor      
      CLOSE crs;

      -- Registra el final de la ejecucin en la tabla de LOGS.      
      rsp:='OK';
      PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);
      
   END IF;

   EXCEPTION
       -- Si no se ha podido abrir registro para el log informa al usuario. 
       WHEN problema_log THEN
         rsp:= 'ERROR. No se puede guardar el log de la transaccin.';
       --Para cualquier otro error informa en la tabla de LOG del error.
       WHEN OTHERS THEN
         rsp:='ERROR ' || SUBSTR(SQLERRM,1,200);
         PKG_CONTROL.ACTUALIZA_LOG_SALIDA (id_log,rsp);

END ACTLZR_EST_GLOBAL;


FUNCTION CAL_BN_AGREGADO (
    e_periodo IN NUMBER
) RETURN NUMBER

AS 
 
    Beneficio_neto NUMBER;
    crs SYS_REFCURSOR;

BEGIN
 
   -- si el periodo es 0 calcula para todas las ventas. Si no calcula para el periodo concreto.
   IF e_periodo = 0 THEN
      SELECT SUM (UD_VENDIDAS * BN_UNITARIO) INTO Beneficio_neto
      FROM VENTAS;
    ELSE
      SELECT SUM (UD_VENDIDAS * BN_UNITARIO) INTO Beneficio_neto
      FROM VENTAS
      WHERE TO_CHAR (FECHA, 'YYYY') = e_periodo;
    END IF;
   
    RETURN Beneficio_neto;
    
    EXCEPTION
     WHEN NO_DATA_FOUND THEN
       Beneficio_neto := 0;
    WHEN OTHERS THEN
       RAISE;

END CAL_BN_AGREGADO ;

END PKG_EST_DIR;

/
spool off;